package com.bryceredd.web;

import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;

import net.minidev.json.JSONArray;
import net.minidev.json.JSONObject;
import net.minidev.json.JSONValue;

import org.apache.http.entity.StringEntity;

import android.content.Context;
import android.graphics.Bitmap;
import android.os.Handler;

import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;

import de.greenrobot.event.EventBus;

public class AsyncRequest {

	public enum Action{
		GET,
		POST,
		PUT,
		DELETE,
		IMAGE
	}
	
	public static Handler handler = null;
	public static Context context = null;
	
	private static ImageLoader imageLoader;
	public static AsyncHttpClient client = new AsyncHttpClient();

	protected StringEntity body;
	protected String url = "";
	public AsyncResponse response;

	public boolean hitCache;
	public Bitmap responseImage; 
	public String responseString;
	private Action action;
	protected Throwable error;
	
	public static void initialize(Handler h, Context c) {
		handler = h;
		context = c;
	} 
	
	public void setBody(StringEntity body) {
		this.body = body;
	}

	public void sendImageRequest(String url, AsyncResponse response) {
		action = Action.IMAGE;
		this.response = response;
		this.url = url;

		if (imageLoader == null)
			imageLoader = new ImageLoader(context);
		
		imageLoader.loadImage(url, handler, new ImageLoadedResponse() {
			protected void imageLoaded(Bitmap bitmap) {
				AsyncRequest.this.responseImage = bitmap;
				AsyncRequest.this.response.success(AsyncRequest.this);
			}

			@Override
			protected void failed() {
				AsyncRequest.this.response.failure(AsyncRequest.this);
			}
		});
	}

	public void putJson(String url, AsyncResponse response) {
		action = Action.PUT;
		this.response = response;
		this.url = url;
		
		client.put(context, url, body, "application/json", handler());
	}
	
	public void deleteJson(String url, AsyncResponse response) {
		action = Action.DELETE;
		this.response = response;
		this.url = url;
		
		client.delete(context, url, handler());
	}
	
	public void postJson(String url, AsyncResponse response) {
		action = Action.POST;
		this.response = response;
		this.url = url;
		
		client.post(context, url, body, "application/json", handler());
	}

	public void getJson(String url, AsyncResponse response) {
		action = Action.GET;
		this.response = response;
		this.url = url;

		client.get(context, url, handler());
	}
	
	private AsyncHttpResponseHandler handler() {
		return new AsyncHttpResponseHandler() {
			@Override
			public void onSuccess(String response) {
				AsyncRequest.this.responseString = response;
				AsyncRequest.this.response.success(AsyncRequest.this);
			}
			public void onFailure(Throwable error, String response) {
				AsyncRequest.this.responseString = response;
				AsyncRequest.this.error = error;
				postErrorToEventBus(error, response);
				AsyncRequest.this.response.failure(AsyncRequest.this);
			}
		};
	}

	public String responseString() {
		return responseString;
	}

	public Bitmap responseImage() {
		return responseImage;
	}

	public JSONArray responseArray() {
		return (JSONArray) JSONValue.parse(responseString);
	}

	public JSONObject responseObject() {
		return (JSONObject) JSONValue.parse(responseString);

	}
	
	public String toString() {
		return "AsyncRequest: "+url;
	}
	
	public static void cancelForContext(Context context) {
		client.cancelRequests(context, true);
	}
	
	public static void clearCache() {
		if(imageLoader != null) 
			imageLoader.clearCache();
	}
	
	public static void clearImageUrlRequest(String url) {
		imageLoader.photosQueue.Clean(url);
	}
	
	public void addHeader(String header, String value){
		client.addHeader(header, value);
	}

	protected void postErrorToEventBus(Throwable error, String response) {
		if(error instanceof UnknownHostException || error instanceof SocketException || error instanceof SocketTimeoutException){
			EventBus.getDefault().post(new NoInternetError(error, response, this, action));
		}
	}
	
}
